/*
 * lst2sym - Simple filter to convert from XA LST output to Oricutron SYM input
 * iss, 2016
 *
 * This source file is absolutely free.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define vmajor  0
#define vminor  1

/*
 * 1k line length should be enough
 */
#define max_linelen 1024
static char line[max_linelen];

#define max_symlen 256
static char sym[max_symlen];

static void hello(void)
{
  fprintf(stderr, "lst2sym v.%d.%d, iss, 2016\n", vmajor, vminor);
}

static void usage(char* prgname)
{
  hello();
  fprintf(stderr, "usage: %s <path_to_input_lst> <path_to_output_sym>\n", prgname);
  exit(-1);
}

static void error(char* errfmt, char* errarg)
{
  hello();
  fprintf(stderr, "error: ");
  fprintf(stderr, errfmt, errarg);
  exit(-2);
}

static int compar(const void *a, const void *b)
{
  const char **ia = (const char **)a;
  const char **ib = (const char **)b;
  return strcmp(*ia, *ib);
}

/*@
 * main
 */
int main(int argc, char* argv[])
{
  int cnt, a;
  char *p, *pp, **arr;

  FILE *fi, *fo;

  if(argc !=3)
    usage(argv[0]);

  fi = fopen(argv[1], "rt");
  if(!fi)
    error("Can't open input file: %s\n", argv[1]);

  fo = fopen(argv[2], "wt");
  if(!fo)
  {
    fclose(fi);
    error("Can't open output file: %s\n", argv[2]);
  }

  for(cnt = 0, arr = NULL; !feof(fi); )
  {
    if(fgets(line, max_linelen, fi))
    {
      p = strchr(line,',');
      if(p)
      {
        *p++ = '\0';
        strncpy(sym, line, max_symlen);
        pp = strchr(p,',');
        if(pp)
        {
          *pp = '\0';
          if(1 == sscanf(p, "%x", &a))
          {
            if(0 < asprintf(&pp, "%.4x %s\r\n", a, sym))
            {
              arr = (char**)realloc(arr, sizeof(char*)*(cnt+1));
              arr[cnt++] = pp;
            }
          }
        }
      }
    }
  }

  if(0 < cnt)
    qsort(arr, cnt, sizeof(char*), compar);

  for(a=0; a<cnt; a++)
  {
    fprintf(fo, arr[a]);
    free(arr[a]);
  }

  if(arr)
    free(arr);

  fprintf(stdout, "%d symbol%s converted.\n", cnt, (1 == cnt)? "":"s");
  fflush(fo);
  fclose(fo);
  fclose(fi);

  return 0;
}
